from typing import Optional, cast

from birdeyepy.utils import BirdEyeApiUrls, BirdEyeRequestParams, IHttp, as_api_args


class Token:
    def __init__(self, http: IHttp) -> None:
        self.http = http

    def list_all(
        self,
        *,
        sort_by: Optional[str] = "v24hUSD",
        sort_type: Optional[str] = "desc",
        offset: Optional[int] = 0,
        limit: Optional[int] = 50,
        min_liquidity: Optional[int] = 50,
    ) -> dict:
        """Get token list of any supported chains.

        :param sort_by:         The field to sort by.
        :param sort_type:       The type of sorting.
        :param offset:          The offset
        :param limit:           The limit
        :param min_liquidity:   The minimum liquidity to check.
        """
        params = {
            "sort_by": sort_by,
            "sort_type": sort_type,
            "offset": offset,
            "limit": limit,
            "min_liquidity": min_liquidity,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.DEFI_TOKEN_LIST, **request)

        return cast(dict, response)

    def security(self, address: str) -> dict:
        """Get token security of any supported chains.

        :param address:     The address of the token.
        """
        params = {"address": address}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.TOKEN_SECURITY, **request)

        return cast(dict, response)

    def overview(self, address: str) -> dict:
        """Get overview of a token.

        :param address:     The address of the token.
        """
        params = {"address": address}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.TOKEN_OVERVIEW, **request)

        return cast(dict, response)

    def creation_info(self, address: str) -> dict:
        """Get creation info of a token.

        :param address:     The address of the token.
        """
        params = {"address": address}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.TOKEN_CREATION_INFO, **request)

        return cast(dict, response)

    def trending(
        self,
        sort_by: str = "rank",
        sort_type: str = "asc",
        offset: int = 0,
        limit: int = 10,
    ) -> dict:
        """Retrieve a dynamic and up-to-date list of trending tokens based on specified sorting criteria.

        :param sort_by:     The field to sort by.
        :param sort_type:   The type of sorting.
        :param offset:      The offset.
        :param limit:       The limit.
        """
        params = {
            "sort_by": sort_by,
            "sort_type": sort_type,
            "offset": offset,
            "limit": limit,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.TOKEN_TRENDING, **request)

        return cast(dict, response)

    def list_all_v2(self) -> dict:
        """This endpoint facilitates the retrieval of a list of tokens on a specified blockchain network. This upgraded version is exclusive to business and enterprise packages. By simply including the header for the requested blockchain without any query parameters, business and enterprise users can get the full list of tokens on the specified blockchain in the URL returned in the response. This removes the need for the limit response of the previous version and reduces the workload of making multiple calls."""
        response = self.http.send(path=BirdEyeApiUrls.TOKEN_LIST_V2, method="POST")

        return cast(dict, response)

    @as_api_args
    def new_listing(
        self,
        time_to: int,
        limit: Optional[int] = None,
        meme_platform_enabled: Optional[bool] = None,
    ) -> dict:
        """Get newly listed tokens of any supported chains.

        :param time_to:                 Specify the end time using Unix timestamps in seconds
        :param limit:                   The limit
        :param meme_platform_enabled:   Enable to receive token new listing from meme platforms (eg: pump.fun). This filter only supports Solana
        """
        params = {
            "time_to": time_to,
        }

        if limit is not None:
            params["limit"] = limit

        if meme_platform_enabled is not None:
            params["meme_platform_enabled"] = meme_platform_enabled

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.TOKEN_NEW_LISTING, **request)

        return cast(dict, response)

    def top_traders(
        self,
        address: str,
        time_frame: Optional[str] = "24h",
        sort_type: Optional[str] = "desc",
        sort_by: Optional[str] = "volume",
        offset: Optional[int] = 0,
        limit: Optional[int] = 10,
    ) -> dict:
        """Get top traders of given token.

        :param address:     The address of the token.
        :param time_frame:  The time frame for the data (e.g., '24h', '7d').
        :param sort_type:   The type of sorting.
        :param sort_by:     The field to sort by.
        :param offset:      The offset.
        :param limit:       The limit.
        """
        params = {
            "address": address,
            "time_frame": time_frame,
            "sort_type": sort_type,
            "sort_by": sort_by,
            "offset": offset,
            "limit": limit,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.TOKEN_TOP_TRADERS, **request)

        return cast(dict, response)

    def all_markets(
        self,
        address: str,
        time_frame: Optional[str] = "24h",
        sort_type: Optional[str] = "desc",
        sort_by: Optional[str] = "liquidity",
        offset: Optional[int] = 0,
        limit: Optional[int] = 10,
    ) -> dict:
        """The API provides detailed information about the markets for a specific cryptocurrency token on a specified blockchain. Users can retrieve data for one or multiple markets related to a single token. This endpoint requires the specification of a token address and the blockchain to filter results. Additionally, it supports optional query parameters such as offset, limit, and required sorting by liquidity or sort type (ascending or descending) to refine the output.

        :param address:     The address of the token.
        :param time_frame:  The time frame for the data (e.g., '24h', '7d').
        :param sort_type:   The type of sorting.
        :param sort_by:     The field to sort by.
        :param offset:      The offset.
        :param limit:       The limit.
        """
        params = {
            "address": address,
            "time_frame": time_frame,
            "sort_type": sort_type,
            "sort_by": sort_by,
            "offset": offset,
            "limit": limit,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.TOKEN_ALL_MARKETS, **request)

        return cast(dict, response)

    def market_metadata_single(self, address: str) -> dict:
        """Get metadata of single token

        :param address:       The address of the token.
        """
        params = {"address": address}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.TOKEN_METADATA_SINGLE, **request)

        return cast(dict, response)

    @as_api_args
    def market_metadata_multiple(self, addresses: str | list[str]) -> dict:
        """Get metadata of multiple tokens

        :param addresses:       The address of the token...can be comma separated string or list of strings.
        """
        params = {"list_address": addresses}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(
            path=BirdEyeApiUrls.TOKEN_METADATA_MULTIPLE, **request
        )

        return cast(dict, response)

    def market_data(self, address: str) -> dict:
        """Get market data of single token.

        :param address:       The address of the token.
        """
        params = {"address": address}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.TOKEN_MARKET_DATA, **request)

        return cast(dict, response)

    def trade_data_single(self, address: str) -> dict:
        """Get trade data of single token

        :param addresses:       The address of the token.
        """
        params = {"address": address}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(
            path=BirdEyeApiUrls.TOKEN_TRADE_DATA_SINGLE, **request
        )

        return cast(dict, response)

    @as_api_args
    def trade_data_multiple(self, addresses: str | list[str]) -> dict:
        """Get trade data of multiple tokens.

        :param addresses:       The address of the token...can be comma separated string or list of strings.
        """
        params = {"list_address": addresses}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(
            path=BirdEyeApiUrls.TOKEN_TRADE_DATA_MULTIPLE, **request
        )

        return cast(dict, response)

    def holder(self, address: str, offset: int = 0, limit: int = 100) -> dict:
        """Get top holder list of the given token.

        :param address:     The address of the token.
        :param offset:      The offset.
        :param limit:       The limit.
        """
        params = {"address": address, "offset": offset, "limit": limit}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.TOKEN_HOLDER, **request)

        return cast(dict, response)



from typing import Optional, cast

from birdeyepy.utils import BirdEyeApiUrls, BirdEyeRequestParams, IHttp


class Trader:
    def __init__(self, http: IHttp) -> None:
        self.http = http

    def gainers_losers(
        self,
        *,
        time_frame: Optional[str] = "1W",
        sort_by: Optional[str] = "PnL",
        sort_type: Optional[str] = "desc",
        offset: Optional[int] = 0,
        limit: Optional[int] = 10,
    ) -> list:
        """The API provides detailed information top gainers/losers

        :param time_frame: The time frame for the data (e.g., '24h', '7d').
        :param limit:      The number of results to return.
        """
        params = {
            "time_frame": time_frame,
            "sort_by": sort_by,
            "sort_type": sort_type,
            "offset": offset,
            "limit": limit,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.TRADER_GAINERS_LOSERS, **request)

        return cast(list, response)

    def seek_by_time(
        self,
        *,
        address: str,
        tx_type: Optional[str] = "swap",
        sort_type: Optional[str] = "desc",
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        before_time: Optional[int] = 0,
        after_time: Optional[int] = 0,
    ) -> dict:
        """Get list of trades of a trader with time bound option.

        :param address:             The address of the trader.
        :param tx_type:             The type of transaction.
        :param offset:              The offset.
        :param limit:               The limit.
        :param before_time:         Specify the time seeked before using Unix timestamps in seconds
        :param after_time:          Specify the time seeked after using Unix timestamps in seconds
        """
        params = {
            "address": address,
            "tx_type": tx_type,
            "sort_type": sort_type,
            "offset": offset,
            "limit": limit,
            "before_time": before_time,
            "after_time": after_time,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.TRADER_SEEK_BY_TIME, **request)

        return cast(dict, response)



class SimpleEnum:
    @classmethod
    def all(cls) -> list[str]:
        return [v for k, v in cls.__dict__.items() if not k.startswith("__")]


class BirdEyeApiUrls:
    # DEFI
    DEFI_PRICE = "defi/price"
    DEFI_TOKEN_LIST = "defi/tokenlist"
    DEFI_HISTORY_PRICE = "defi/history_price"
    DEFI_SUPPORTED_NETWORKS = "defi/networks"
    DEFI_PRICE_MULTIPLE = "defi/multi_price"
    DEFI_HISTORY_PRICE_BY_UNIX = "defi/historical_price_unix"
    DEFI_TRADES_TOKEN = "defi/txs/token"
    DEFI_TRADES_PAIR = "defi/txs/pair"
    DEFI_TRADES_TOKEN_BY_TIME = "defi/txs/token/seek_by_time"
    DEFI_TRADES_PAIR_BY_TIME = "defi/txs/pair/seek_by_time"
    DEFI_OHLCV = "defi/ohlcv"
    DEFI_OHLCV_PAIR = "defi/ohlcv/pair"
    DEFI_VOLUME_SINGLE = "defi/price_volume/single"

    # TRADER
    TRADER_GAINERS_LOSERS = "trader/gainers-losers"
    TRADER_SEEK_BY_TIME = "trader/txs/seek_by_time"

    # TOKEN
    TOKEN_SECURITY = "defi/token_security"
    TOKEN_OVERVIEW = "defi/token_overview"
    TOKEN_CREATION_INFO = "defi/token_creation_info"
    TOKEN_TRENDING = "defi/token_trending"
    TOKEN_LIST_V2 = "/defi/v2/tokens/all"
    TOKEN_NEW_LISTING = "defi/v2/tokens/new_listing"
    TOKEN_TOP_TRADERS = "defi/v2/tokens/top_traders"
    TOKEN_ALL_MARKETS = "/defi/v2/markets"
    TOKEN_METADATA_SINGLE = "defi/v3/token/meta-data/single"
    TOKEN_METADATA_MULTIPLE = "defi/v3/token/meta-data/multiple"
    TOKEN_MARKET_DATA = "defi/v3/token/market-data"
    TOKEN_HOLDER = "defi/v3/token/holder"
    TOKEN_TRADE_DATA_SINGLE = "defi/v3/token/trade-data/single"
    TOKEN_TRADE_DATA_MULTIPLE = "defi/v3/token/trade-data/multiple"

    # WALLET (Beta)
    WALLET_SUPPORTED_NETWORKS = "v1/wallet/list_supported_chain"
    WALLET_PORTFOLIO = "v1/wallet/token_list"
    WALLET_PORTFOLIO_MULTICHAIN = "v1/wallet/multichain_token_list"
    WALLET_TOKEN_BALANCE = "v1/wallet/token_balance"
    WALLET_TRANSACTION_HISTORY = "v1/wallet/tx_list"
    WALLET_TRANSACTION_HISTORY_MULTICHAIN = "v1/wallet/multichain_tx_list"
    WALLET_TRANSACTION_SIMULATION = "v1/wallet/simulate"

    # PAIR
    PAIR_OVERVIEW_MULTIPLE = "defi/v3/pair/overview/multiple"
    PAIR_OVERVIEW_SINGLE = "defi/v3/pair/overview/single"


class BirdEyeChainEnum(SimpleEnum):
    # Solana
    SOLANA = "solana"
    ETHEREUM = "ethereum"
    BSC = "bsc"
    AVALANCHE = "avalanche"
    ARBITRUM = "arbitrum"
    OPTIMISM = "optimism"
    POLYGON = "polygon"
    BASE = "base"
    ZKSYNC = "zksync"
    SUI = "sui"



from birdeyepy.resources import RESOURCE_MAP
from birdeyepy.utils import (
    BASE_BIRD_EYE_API_URL,
    BirdEyeChainEnum,
    BirdEyeClientError,
    RequestsClient,
)


__version__ = "0.0.8"


class BirdEye:
    """API Client for BirdEye

    :param api_key:         The API key for the BirdEye. See here https://docs.birdeye.so/docs/authentication-api-keys
    :param chain:           The chain to use. Defaults to 'solana'
    """

    def __init__(self, api_key: str, chain: str = BirdEyeChainEnum.SOLANA) -> None:
        if chain not in BirdEyeChainEnum.all():
            raise BirdEyeClientError(f"Invalid chain: {chain}")

        _http = RequestsClient(
            base_url=BASE_BIRD_EYE_API_URL,
            headers={
                "x-chain": chain,
                "X-API-KEY": api_key,
                "User-Agent": f"birdeyepy/v{__version__}",
            },
        )

        for resource_name, resource_class in RESOURCE_MAP.items():
            setattr(self, resource_name, resource_class(http=_http))



from typing import Optional, cast

from birdeyepy.utils import BirdEyeApiUrls, BirdEyeRequestParams, IHttp


class Wallet:
    """The api is still in beta and may change in the future."""

    def __init__(self, http: IHttp) -> None:
        self.http = http

    def supported_networks(self) -> dict:
        """Get supported networks."""
        response = self.http.send(path=BirdEyeApiUrls.WALLET_SUPPORTED_NETWORKS)

        return cast(dict, response)

    def portfolio(self, wallet: str) -> dict:
        """Get portfolio of a wallet.

        :param address:     The address of the wallet.
        """
        params = {"wallet": wallet}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.WALLET_PORTFOLIO, **request)

        return cast(dict, response)

    def portfolio_multichain(self, wallet: str) -> dict:
        """Get portfolio of a wallet across multiple chains.

        :param address:     The address of the wallet.
        """
        params = {"wallet": wallet}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(
            path=BirdEyeApiUrls.WALLET_PORTFOLIO_MULTICHAIN, **request
        )

        return cast(dict, response)

    def token_balance(self, wallet: str, token_address: str) -> dict:
        """Get token balance of a wallet.

        :param wallet:          The address of the wallet.
        :param token_address:   The address of the token.
        """
        params = {"wallet": wallet, "address": token_address}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.WALLET_TOKEN_BALANCE, **request)

        return cast(dict, response)

    def transaction_history(
        self, wallet: str, limit: Optional[int] = 100, before: Optional[str] = None
    ) -> dict:
        """Get transaction history of a wallet.

        :param wallet:      The address of the wallet.
        :param limit:       The number of results to return.
        :param before:      A transaction hash to traverse starting from. Only works with Solana
        """
        params = {"wallet": wallet, "limit": limit}

        if before is not None:
            params["before"] = before

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(
            path=BirdEyeApiUrls.WALLET_TRANSACTION_HISTORY, **request
        )

        return cast(dict, response)

    def transaction_history_multichain(self, wallet: str) -> dict:
        """Get transaction history of a wallet across multiple chains.

        :param wallet:      The address of the wallet.
        """
        params = {"wallet": wallet}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(
            path=BirdEyeApiUrls.WALLET_TRANSACTION_HISTORY_MULTICHAIN, **request
        )

        return cast(dict, response)

    def transaction_simulation(
        self, from_address: str, to_address: str, value: str, data: str
    ) -> dict:
        """Simulate a transaction.

        :param from_address:    The address of the sender.
        :param to_address:      The address of the receiver.
        :param value:           The value to send.
        :param data:            The data to send.
        """
        params = {"from": from_address, "to": to_address, "value": value, "data": data}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(
            method="POST", path=BirdEyeApiUrls.WALLET_TRANSACTION_SIMULATION, **request
        )

        return cast(dict, response)



from typing import cast

from birdeyepy.utils import BirdEyeApiUrls, BirdEyeRequestParams, IHttp, as_api_args


class Pair:
    def __init__(self, http: IHttp) -> None:
        self.http = http

    @as_api_args
    def overview_multiple(
        self,
        *,
        list_address: list[str],
    ) -> dict:
        """Get overview of multiple pairs.

        :param list_address:        A list of addresses
        """
        params = {"list_address": list_address}
        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.PAIR_OVERVIEW_MULTIPLE, **request)

        return cast(dict, response)

    @as_api_args
    def overview_single(
        self,
        *,
        address: str,
    ) -> dict:
        """Get overview of single pair

        :param list_address:        A list of addresses
        """
        params = {"address": address}
        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.PAIR_OVERVIEW_SINGLE, **request)

        return cast(dict, response)



from typing import Optional, cast

from birdeyepy.utils import (
    DEFAULT_SOL_ADDRESS,
    BirdEyeApiUrls,
    BirdEyeRequestParams,
    IHttp,
    as_api_args,
)


class DeFi:
    def __init__(self, http: IHttp) -> None:
        self.http = http

    @as_api_args
    def price(
        self,
        *,
        address: Optional[str] = DEFAULT_SOL_ADDRESS,
        check_liquidity: Optional[int] = 100,
        include_liquidity: Optional[bool] = None,
    ) -> list:
        """Get price update of a token.

        :param address:             The address of the token.
        :param check_liquidity:     The minimum liquidity to check.
        :param include_liquidity:   Include liquidity in the response.
        """
        params = {"address": address, "check_liquidity": check_liquidity}

        if include_liquidity is not None:
            params["include_liquidity"] = include_liquidity

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.DEFI_PRICE, **request)

        return cast(list, response)

    def history(
        self,
        *,
        time_from: int,
        time_to: int,
        address: Optional[str] = DEFAULT_SOL_ADDRESS,
        address_type: Optional[str] = "token",
        type_in_time: Optional[str] = "15m",
    ) -> dict:
        """Get historical price line chart of a token.

        :param time_from:       Specify the start time using Unix timestamps in seconds
        :param time_to:         Specify the end time using Unix timestamps in seconds
        :param address:         The address of the token.
        :param address_type:    The type of the address...defaults to 'token'
        :param type_in_time:    The type of time...defaults to '15m'
        """
        params = {
            "address": address,
            "address_type": address_type,
            "type": type_in_time,
            "time_from": time_from,
            "time_to": time_to,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.DEFI_HISTORY_PRICE, **request)

        return cast(dict, response)

    def supported_networks(self) -> dict:
        """Get a list of all supported networks."""
        response = self.http.send(path=BirdEyeApiUrls.DEFI_SUPPORTED_NETWORKS)

        return cast(dict, response)

    @as_api_args
    def price_multiple(
        self,
        *,
        addresses: list | str,
        check_liquidity: Optional[int] = 100,
        include_liquidity: Optional[bool] = None,
    ) -> dict:
        """Get price updates of multiple tokens in a single API call. Maximum 100 tokens

        :param addresses:           The addresses of the tokens.
        :param check_liquidity:     The minimum liquidity to check.
        :param include_liquidity:   Include liquidity in the response.
        """
        params = {"list_address": addresses, "check_liquidity": check_liquidity}

        if include_liquidity is not None:
            params["include_liquidity"] = include_liquidity

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.DEFI_PRICE_MULTIPLE, **request)

        return cast(dict, response)

    def history_by_unix(self, *, address: str, unixtime: int) -> dict:
        """Get historical price of a token at a specific Unix time.

        :param address:     The address of the token.
        :param unixtime:    The Unix time.
        """
        params = {"address": address, "time": unixtime}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(
            path=BirdEyeApiUrls.DEFI_HISTORY_PRICE_BY_UNIX, **request
        )

        return cast(dict, response)

    def trades_token(
        self,
        *,
        address: Optional[str] = DEFAULT_SOL_ADDRESS,
        tx_type: Optional[str] = "swap",
        sort_type: Optional[str] = "desc",
        offset: Optional[int] = 0,
        limit: Optional[int] = 50,
    ) -> dict:
        """Get trades of a token.

        :param address:     The address of the token.
        :param tx_type:     The type of transaction.
        :param sort_type:   The type of sorting.
        :param limit:       The limit.
        """
        params = {
            "address": address,
            "tx_type": tx_type,
            "sort_type": sort_type,
            "offset": offset,
            "limit": limit,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.DEFI_TRADES_TOKEN, **request)

        return cast(dict, response)

    def trades_pair(
        self,
        *,
        address: str,
        tx_type: Optional[str] = "swap",
        sort_type: Optional[str] = "desc",
        offset: Optional[int] = 0,
        limit: Optional[int] = 50,
    ) -> dict:
        """Get list of trades of a certain pair or market.

        :param address:     The address of the token.
        :param tx_type:     The type of transaction.
        :param sort_type:   The type of sorting.
        :param limit:       The limit.
        """
        params = {
            "address": address,
            "tx_type": tx_type,
            "sort_type": sort_type,
            "offset": offset,
            "limit": limit,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.DEFI_TRADES_PAIR, **request)

        return cast(dict, response)

    def trades_token_by_time(
        self,
        *,
        address: Optional[str] = DEFAULT_SOL_ADDRESS,
        tx_type: Optional[str] = "swap",
        sort_type: Optional[str] = "desc",
        offset: Optional[int] = 0,
        limit: Optional[int] = 50,
        before_time: Optional[int] = 0,
        after_time: Optional[int] = 0,
    ) -> dict:
        """Get list of trades of a token with time bound option."""
        params = {
            "address": address,
            "tx_type": tx_type,
            "sort_type": sort_type,
            "offset": offset,
            "limit": limit,
            "before_time": before_time,
            "after_time": after_time,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(
            path=BirdEyeApiUrls.DEFI_TRADES_TOKEN_BY_TIME, **request
        )

        return cast(dict, response)

    def trades_pair_by_time(
        self,
        *,
        address: Optional[str] = DEFAULT_SOL_ADDRESS,
        tx_type: Optional[str] = "swap",
        sort_type: Optional[str] = "desc",
        offset: Optional[int] = 0,
        limit: Optional[int] = 50,
        before_time: Optional[int] = 0,
        after_time: Optional[int] = 0,
    ) -> dict:
        """Get list of trades of a certain pair or market with time bound option."""
        params = {
            "address": address,
            "tx_type": tx_type,
            "sort_type": sort_type,
            "offset": offset,
            "limit": limit,
            "before_time": before_time,
            "after_time": after_time,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(
            path=BirdEyeApiUrls.DEFI_TRADES_PAIR_BY_TIME, **request
        )

        return cast(dict, response)

    def ohlcv(
        self,
        *,
        address: str,
        time_from: int,
        time_to: int,
        type_in_time: Optional[str] = "15m",
    ) -> dict:
        """Get OHLCV price of a token.

        :param address:         The address of the token.
        :param time_from:       Specify the start time using Unix timestamps in seconds
        :param time_to:         Specify the end time using Unix timestamps in seconds
        :param type_in_time:    The type of time...defaults to '15m'
        """
        params = {
            "address": address,
            "time_from": time_from,
            "time_to": time_to,
            "type": type_in_time,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.DEFI_OHLCV, **request)

        return cast(dict, response)

    def ohlcv_pair(
        self,
        *,
        address: str,
        time_from: int,
        time_to: int,
        type_in_time: Optional[str] = "15m",
    ) -> dict:
        """Get OHLCV price of a pair.

        :param address:         The address of the token.
        :param time_from:       Specify the start time using Unix timestamps in seconds
        :param time_to:         Specify the end time using Unix timestamps in seconds
        :param type_in_time:    The type of time...defaults to '15m'
        """
        params = {
            "address": address,
            "time_from": time_from,
            "time_to": time_to,
            "type": type_in_time,
        }

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.DEFI_OHLCV_PAIR, **request)

        return cast(dict, response)

    def volume_price_single(
        self, *, address: str, type_in_time: Optional[str] = "24h"
    ) -> dict:
        """Get volume and price of a token.

        :param address:         The address of the token.
        :param type_in_time:    The type of time...defaults to '24h'
        """
        params = {"address": address, "type": type_in_time}

        request: BirdEyeRequestParams = {"params": params}
        response = self.http.send(path=BirdEyeApiUrls.DEFI_VOLUME_SINGLE, **request)

        return cast(dict, response)



import copy
import logging
from typing import Any, Optional

import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from birdeyepy.utils.exceptions import BirdEyeRequestError
from birdeyepy.utils.interfaces import IHttp


logger = logging.getLogger(__name__)
default_headers = {"Content-Type": "application/json"}


def _get_retry() -> Retry:
    return Retry(
        total=3,
        backoff_factor=2,
        status_forcelist=[429, 500, 502, 503, 504],
        allowed_methods=["HEAD", "GET", "OPTIONS"],
    )


def _get_headers(headers: Optional[dict] = None) -> dict:
    _headers = copy.deepcopy(default_headers)

    if headers is not None:
        _headers.update(**headers)

    return _headers


def _get_session(headers: Optional[dict] = None) -> requests.Session:
    s = requests.Session()
    adapter = HTTPAdapter(max_retries=_get_retry())
    _headers = _get_headers(headers)

    s.mount("https://", adapter)
    s.mount("http://", adapter)

    s.headers.update(_headers)

    return s


class RequestsClient(IHttp):
    def __init__(
        self, base_url: Optional[str] = "", headers: Optional[dict] = None
    ) -> None:
        self.base_url = base_url
        self.s = _get_session(headers)

    def send(self, path: str, method: str = "get", **extra: Any) -> dict | list:
        url = f"{self.base_url}{path}"
        headers = extra.pop("headers", {})

        self.s.headers.update(headers)

        req_to_call = getattr(self.s, method.lower())

        try:
            response = req_to_call(url, **extra)
            response.raise_for_status()
        except requests.exceptions.HTTPError as http_err:
            logger.error(f"HTTP error occurred: {http_err}", exc_info=True)
            raise BirdEyeRequestError(
                message=str(http_err), response=http_err.response
            ) from None
        except Exception as err:
            logger.error(f"Other error occurred: {err}")
            raise

        return response.json()  # type: ignore



from requests import Response


class BaseBirdEyeError(Exception):
    """Base class for exceptions in this module."""

    pass


class BirdEyeRequestError(BaseBirdEyeError):
    def __init__(self, message: str, response: Response) -> None:
        self.message = message

        super().__init__(message)

        self.response = response


class BirdEyeClientError(BaseBirdEyeError):
    pass



from birdeyepy.utils.constants import BASE_BIRD_EYE_API_URL, DEFAULT_SOL_ADDRESS
from birdeyepy.utils.enums import BirdEyeApiUrls, BirdEyeChainEnum
from birdeyepy.utils.exceptions import BirdEyeClientError
from birdeyepy.utils.helpers import as_api_args
from birdeyepy.utils.http import RequestsClient
from birdeyepy.utils.interfaces import IHttp
from birdeyepy.utils.types import BirdEyeRequestParams


__all__ = [
    "IHttp",
    "BASE_BIRD_EYE_API_URL",
    "DEFAULT_SOL_ADDRESS",
    "BirdEyeApiUrls",
    "BirdEyeChainEnum",
    "BirdEyeRequestParams",
    "IHttp",
    "RequestsClient",
    "as_api_args",
    "BirdEyeClientError",
]
